import 'dart:io';

import 'package:flutter/cupertino.dart';
import 'package:flutter/services.dart';
import '../utils/fx_utils.dart';
import 'native_model.dart';

abstract class NativeState<T extends StatefulWidget> extends State<T>{
  late MethodChannel methodChannel;
  late String channelName;

  Future<T?>  invokeNativeMethod(String method, [ dynamic arguments ]){
    return methodChannel.invokeMethod(method, arguments);
  }

  void messageFromNative(MethodCall? call);
  String providePageRoute();

  T get auth_widget => widget;

  @override
  void initState() {
    super.initState();
    channelName = "message_channel_${providePageRoute}_$hashCode";
    methodChannel = MethodChannel(channelName);
    _observeNativeData();
    NativeModel.instance.newPageCreate(providePageRoute(), channelName);
  }
  
  @override
  void dispose() {
    super.dispose();
    NativeModel.instance.onPageDestory(providePageRoute(), channelName);
  }

  void onBackPress() async{
    var isPoped = await Navigator.maybePop(context);
    if(!isPoped){
      if (Platform.isAndroid) {
        SystemNavigator.pop();
      } else {
        NativeModel.instance.onBackPress(providePageRoute());
      }
    }
  }

  void _observeNativeData() {
    methodChannel.setMethodCallHandler((call) async {
      FxUtils.log("received native message");
      FxUtils.log(call);
      messageFromNative(call);
    });
  }

}