import 'dart:convert';

import 'package:flutter/cupertino.dart';
import 'package:flutter/services.dart';
import 'package:lib_common_fx/lib_common_fx.dart';

import 'native_model.dart';

class NativeNavigator extends StatefulWidget {
  const NativeNavigator({Key? key, this.defalutContent, required this.pageBuilder}) : super(key: key);
  final Widget? defalutContent;
  final Widget? Function(String? pageRoute, String? pageParams) pageBuilder;

  @override
  State<StatefulWidget> createState() {
    return NativeNavigatorState();
  }
}

class NativeNavigatorState extends State<NativeNavigator> {
  final MethodChannel _methodChannel = const MethodChannel('com.kachexiongdi.app/InitialMessageChannel');
  Widget? contentWidget;

  @override
  void initState() {
    contentWidget = widget.defalutContent;
    NativeModel.instance.init(_methodChannel);
    _observeNativeData();
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return contentWidget ?? Container();
  }

  void _observeNativeData() {
    _methodChannel.setMethodCallHandler((call) async {
      FxUtils.log("MainPage method: " + call.method + " arguments: " + (call.arguments ?? " no data"));
      if (call.method == "attachNewPage") {
        if (call.arguments is String) {
          setState(() {
            contentWidget = _genTargetWidget(call) ?? const CenterText("找不到对应页面, 确认pageRoute是否正确");
          });
        }
      } else if (call.method == "navigatorNewPage") {
        if (call.arguments is String) {
          Navigator.push(
              context,
              PageRouteBuilder(
                  pageBuilder: (context, animation1, animation2) {
                    return _genTargetWidget(call) ?? const CenterText("找不到对应页面, 确认pageRoute是否正确");
                  },
                  transitionDuration: Duration.zero));
        }
      }else if(call.method == "navigatorReplaceNewPage"){
        if (call.arguments is String) {
          Navigator.pushReplacement(
              context,
              PageRouteBuilder(
                  pageBuilder: (context, animation1, animation2) {
                    return _genTargetWidget(call) ?? const CenterText("找不到对应页面, 确认pageRoute是否正确");
                  },
                  transitionDuration: Duration.zero));
        }
      }
    });
  }

  Widget? _genTargetWidget(MethodCall call) {
    Map<String, dynamic> data = jsonDecode(call.arguments);
    String? pageRoute = data["flutter_page_route"];
    String? pageParams = data["flutter_page_init_params"];
    return widget.pageBuilder(pageRoute, pageParams);
  }
}
