import 'package:dio/dio.dart';
import 'package:lib_common_fx/utils/log_util.dart';

class LogInterceptor extends Interceptor {
  LogInterceptor(this.logEnable);

  final String _tag = "NetWork";
  bool logEnable;

  @override
  void onError(DioError err, ErrorInterceptorHandler handler) {
    printLog("<-- Error ${err.toString()}");
    handler.next(err);
  }

  @override
  void onRequest(RequestOptions options, RequestInterceptorHandler handler) {
    printLog("--> ${options.method} ${options.uri}");
    printLog('--> ${options.headers}');
    printLog("--> body ${options.data}");
    handler.next(options);
  }

  @override
  void onResponse(Response response, ResponseInterceptorHandler handler) {
    printLog("<-- ${response.statusCode} ${response.toString()}");
    handler.next(response);
  }

  void printLog(String msg) {
    if (logEnable) {
      Log.d(tag: _tag, msg: msg);
    }
  }
}
