import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:lib_common_fx/lib_common_fx.dart';

import '../utils/image_utils.dart';

enum ItemStyle {
  arrowAction,
  switchAction,
  redDotAction,
  deleteAction,
  editAction,
}

class EditCommonUi {
  EditCommonUi._();

  @Deprecated('See /lib/widget/future_builder.dart'
      'please use [FxFutureBuilder]')
  static Widget buildLoadingContent(Future? future, Widget content) {
    return FutureBuilder(
      future: future,
      builder: (context, snapshot) {
        if (snapshot.hasError) {
          return Column(
            children: [
              Expanded(
                child: Container(
                  width: double.infinity,
                  decoration: const BoxDecoration(
                      borderRadius: BorderRadius.only(
                          topLeft: Radius.circular(18),
                          topRight: Radius.circular(18)),
                      color: FXColor.main_black6),
                  child: Column(
                    children: [
                      const SizedBox(height: 180),
                      ImageUtil.assetImage("images/icon_net_error.png",
                          width: 300, height: 200),
                      const SizedBox(height: 12),
                      const Text("网络错误"),
                    ],
                  ),
                ),
              ),
            ],
          );
        } else if (snapshot.hasData) {
          return content;
        } else {
          return const Center(
              child: CircularProgressIndicator(color: FXColor.main_blue));
        }
      },
    );
  }

  /**
   * digitsOnly: 数字，只能是整数
   * numOnly: 数字,包括小数
   * intRange:整数位最大长度，默认为10
   */
  static Widget buildInputItem(
      {String? title,
      String? subTitle,
      Color? titleColor,
      Color? bgColor,
      String? unitName,
      String? inputHint,
      String? text,
      TextStyle? style,
      TextStyle? hintStyle,
      EdgeInsetsGeometry? padding,
      bool setPadding = false,
      Function(String value)? onchanged,
      bool enable = true,
      int? decimalRange,
      int integerRange = 10,
      double? height = 48,
      List<TextInputFormatter>? formatter,
      TextEditingController? controller,
      bool digitsOnly = false,
      bool numOnly = false,
      bool hasHorizontalPadding = true,
      bool isDivide = false,
      bool showRedStar = false}) {
    return Column(
      children: [
        Container(
          color: bgColor ?? FXColor.main_white,
          padding: setPadding
              ? padding
              : hasHorizontalPadding
                  ? const EdgeInsets.all(12)
                  : const EdgeInsets.symmetric(vertical: 12),
          height: height,
          child: Row(
            children: [
              FXText(title ?? "",
                  style: FXText.content
                      .apply(color: titleColor ?? FXColor.main_black2)),
              if (showRedStar)
                ImageUtil.assetImage("images/ic_redstar.png",
                    width: 15, height: 15),
              const SizedBox(width: 4),
              FXText(subTitle ?? "",
                  style: FXText.content
                      .apply(color: FXColor.main_black4, fontSizeDelta: -2)),
              const SizedBox(width: 8),
              Expanded(
                child: TextFormField(
                    controller: text == null ? controller : null,
                    enabled: enable,
                    initialValue: text,
                    textAlign: TextAlign.right,
                    keyboardType: numOnly || digitsOnly
                        ? const TextInputType.numberWithOptions(decimal: true)
                        : null,
                    inputFormatters: [
                      if (digitsOnly) FilteringTextInputFormatter.digitsOnly,
                      if (numOnly) ...[
                        Remove0PrefixFormatter(),
                        DecimalTextInputFormatter(
                            decimalRange: decimalRange ?? 2,
                            integerRange: integerRange),
                        FilteringTextInputFormatter.allow(RegExp("[0-9.]")),
                        ...?formatter,
                      ]
                    ],
                    decoration: InputDecoration(
                        isCollapsed: true,
                        border: InputBorder.none,
                        hintText: inputHint,
                        hintStyle: hintStyle ??
                            FXText.content.apply(color: FXColor.main_black4)),
                    onChanged: onchanged,
                    style: style),
              ),
              if (unitName != null && unitName.isNotEmpty) ...[
                const SizedBox(
                  width: 4,
                ),
                FXText(unitName,
                    style: FXText.content.apply(color: FXColor.main_black1)),
              ],
            ],
          ),
        ),
        if (isDivide)
          EditCommonUi.buildDividerItem(
              isMargin: hasHorizontalPadding ? true : false),
      ],
    );
  }

  static Widget buildSelectorItem(
      {String? title,
      String? hintText,
      String? text,
      Function()? onClick,
      Color? textColor,
      bool showArrow = true,
      bool isRoundRect = false,
      bool isDivide = false,
      bool hasHorizontalPadding = true,
      bool showRedStar = false}) {
    return GestureDetector(
      behavior: HitTestBehavior.translucent,
      onTap: onClick,
      child: Column(
        children: [
          Container(
            color: isRoundRect ? null : FXColor.main_white,
            decoration: isRoundRect
                ? BoxDecoration(
                    borderRadius: BorderRadius.circular(10),
                    color: FXColor.main_white)
                : null,
            padding: hasHorizontalPadding
                ? const EdgeInsets.all(12)
                : const EdgeInsets.symmetric(vertical: 12),
            height: 48,
            child: Row(
              children: [
                FXText(title ?? "",
                    style: FXText.content.apply(color: FXColor.main_black2)),
                if (showRedStar)
                  ImageUtil.assetImage("images/ic_redstar.png",
                      width: 15, height: 15),
                const SizedBox(width: 8),
                Expanded(
                  child: FXText(
                      !FxUtils.isTextEmpty(text) ? text! : (hintText ?? ""),
                      textAlign: TextAlign.right,
                      maxLines: 1,
                      style: FXText.content.apply(
                          color: !FxUtils.isTextEmpty(text)
                              ? (textColor ?? FXColor.main_black1)
                              : FXColor.main_black4)),
                ),
                if (showArrow)
                  ImageUtil.assetImage("images/icon_right_arrow.png",
                      width: 16, height: 16)
              ],
            ),
          ),
          if (isDivide)
            EditCommonUi.buildDividerItem(
                isMargin: hasHorizontalPadding ? true : false),
        ],
      ),
    );
  }

  /// radius 10方圆角 24半圆角
  static Widget buildConfirmButtonItem(
      {String text = "",
      String image = "",
      bool enable = true,
      double margin = 12,
      double radius = 24,
      Function()? onClick}) {
    return GestureDetector(
      behavior: HitTestBehavior.translucent,
      onTap: enable ? onClick : null,
      child: Container(
          alignment: Alignment.center,
          width: double.infinity,
          height: 48,
          margin: EdgeInsets.symmetric(horizontal: margin),
          decoration: BoxDecoration(
              borderRadius: BorderRadius.circular(radius),
              color: enable ? FXColor.main_blue : FXColor.main_light_blue),
          child: Row(
            mainAxisAlignment: MainAxisAlignment.center,
            children: [
              if (image.isNotEmpty) ...[
                Opacity(
                    opacity: enable ? 1 : 0.5,
                    child: Image.asset(
                      image,
                      width: 16,
                      height: 16,
                    )),
                const SizedBox(
                  width: 8,
                ),
              ],
              CenterText(text,
                  style: CenterText.content.apply(
                      color: FXColor.main_white.withOpacity(enable ? 1 : 0.5),
                      fontSizeDelta: 2))
            ],
          )),
    );
  }

  static Widget buildDividerItem({bool isMargin = true}) {
    return Container(
      color: FXColor.main_black5,
      margin: EdgeInsets.symmetric(horizontal: isMargin ? 12 : 0),
      height: 0.5,
    );
  }

  static Widget buildCardLayout(List<Widget> widgetList,
      {bool hasHorizontalPadding = true,
      bool marginTopEnable = true,
      bool hasHorizontalMarging = true}) {
    return Card(
      clipBehavior: Clip.antiAlias,
      elevation: 0.0,
      margin: EdgeInsets.only(
          left: hasHorizontalMarging ? 12 : 0,
          right: hasHorizontalMarging ? 12 : 0,
          top: marginTopEnable ? 8 : 0),
      shape: const RoundedRectangleBorder(
          borderRadius: BorderRadius.all(Radius.circular(10.0))),
      child: Container(
        color: FXColor.main_white,
        width: double.infinity,
        padding: hasHorizontalPadding
            ? const EdgeInsets.only(left: 12, right: 12)
            : null,
        // decoration: BoxDecoration(borderRadius: BorderRadius.circular(10), color: FXColor.main_white),
        child: Column(
          crossAxisAlignment: CrossAxisAlignment.start,
          children: [...widgetList],
        ),
      ),
    );
  }

/*radius 10方圆角 24半圆角*/
  static Widget buildTwoButtonItem(
      {String? leftMessage,
      Function? leftFunc,
      String? rightMessage,
      Function? rightFunc,
      bool hasBorder = false,
      double radius = 10}) {
    return Container(
      margin: const EdgeInsets.only(top: 8, bottom: 16),
      child: Row(
        children: [
          if (!FxUtils.isTextEmpty(leftMessage))
            GestureDetector(
              onTap: () {
                leftFunc?.call();
              },
              child: Container(
                  height: 48,
                  width: 136,
                  alignment: Alignment.center,
                  decoration: BoxDecoration(
                    border: Border.all(
                        color: hasBorder
                            ? FXColor.main_blue
                            : FXColor.main_blue.withOpacity(0),
                        width: 0.5),
                    borderRadius:
                        BorderRadius.horizontal(left: Radius.circular(radius)),
                    color: FXColor.main_white,
                  ),
                  child: Text(leftMessage ?? "",
                      style: FXText.content
                          .apply(color: FXColor.main_blue, fontSizeDelta: 2))),
            ),
          Expanded(
            child: GestureDetector(
              onTap: () {
                rightFunc?.call();
              },
              child: Container(
                  height: 48,
                  width: 136,
                  alignment: Alignment.center,
                  decoration: BoxDecoration(
                      borderRadius: BorderRadius.horizontal(
                          left: (FxUtils.isTextEmpty(leftMessage)
                              ? Radius.circular(radius)
                              : Radius.zero),
                          right: Radius.circular(radius)),
                      color: FXColor.main_blue),
                  child: Text(rightMessage ?? "",
                      style: FXText.content
                          .apply(color: FXColor.main_white, fontSizeDelta: 2))),
            ),
          ),
        ],
      ),
    );
  }

  static Widget buildSingleSelectionWidget(String title,
      {bool showRedStart = false,
      List<String> selectItems = const [],
      Function(int inex)? onSelect,
      int? selectIndex}) {
    return Container(
      height: 48,
      color: Colors.white,
      padding: const EdgeInsets.only(left: 12, right: 12),
      child: Row(
        crossAxisAlignment: CrossAxisAlignment.center,
        children: [
          FXText(title,
              style: FXText.content.apply(color: FXColor.main_black2)),
          if (showRedStart)
            ImageUtil.assetImage("images/ic_redstar.png",
                width: 15, height: 15),
          const Spacer(),
          ...selectItems.map((e) {
            return GestureDetector(
              behavior: HitTestBehavior.translucent,
              onTap: () {
                onSelect?.call(selectItems.indexOf(e));
              },
              child: Row(
                children: [
                  const SizedBox(width: 24, height: double.infinity),
                  Padding(
                    padding: const EdgeInsets.all(5.0),
                    child: ImageUtil.assetImage(
                        (selectIndex == selectItems.indexOf(e))
                            ? "images/ic_trans_price_change_select.png"
                            : "images/ic_trans_price_change_unselect.png",
                        width: 16,
                        height: 16),
                  ),
                  Text(e, style: FXText.content),
                ],
              ),
            );
          }),
        ],
      ),
    );
  }

  static Widget buildNetErrorLayout() {
    return Center(
        child: Column(
      mainAxisAlignment: MainAxisAlignment.center,
      children: [
        ImageUtil.assetImage("images/icon_net_error.png",
            width: 215, height: 134),
        const SizedBox(height: 12),
        FXText("网络错误",
            style: FXText.content
                .apply(color: FXColor.main_black2, fontSizeDelta: -2)),
      ],
    ));
  }

  static Widget buildRadioBoxItem(String title, List<String> radiosTitle,
      ValueChanged<int?> onChanged, int checkValue,
      {bool isDivide = false}) {
    List<Widget> radios = <Widget>[];
    for (var i = 0; i < radiosTitle.length; i++) {
      radios.add(FxRadioButton<int>(
          value: i + 1,
          groupValue: checkValue,
          onChanged: onChanged,
          text: radiosTitle[i]));
    }
    return Column(
      children: [
        Row(children: [
          Padding(
            padding: const EdgeInsets.only(top: 6, bottom: 6),
            child: FXText(title,
                style: FXText.content.apply(color: FXColor.main_black2)),
          ),
          Expanded(
              flex: 1,
              child: Row(
                mainAxisAlignment: MainAxisAlignment.end,
                children: radios,
              )),
        ]),
        if (isDivide) EditCommonUi.buildDividerItem(isMargin: false),
      ],
    );
  }

  static Widget buildShowArrowItem(
      {String? title,
      Function()? onClick,
      Color? textColor,
      bool isRoundRect = false,
      bool isDivide = false,
      bool hasHorizontalPadding = true,
      Widget? rightWidget}) {
    return _buildShowItem(
        title: title,
        onClick: onClick,
        textColor: textColor,
        isRoundRect: isRoundRect,
        isDivide: isDivide,
        hasHorizontalPadding: hasHorizontalPadding,
        rightWidget: ImageUtil.assetImage("images/icon_right_arrow.png",
            width: 16, height: 16));
  }

  static Widget buildShowRedDotItem(
      {String? title,
      String? numberStr,
      Function()? onClick,
      Color? textColor,
      bool isRoundRect = false,
      bool isDivide = false,
      bool hasHorizontalPadding = true,
      Widget? rightWidget}) {
    return _buildShowItem(
        title: title,
        onClick: onClick,
        textColor: textColor,
        isRoundRect: isRoundRect,
        isDivide: isDivide,
        hasHorizontalPadding: hasHorizontalPadding,
        rightWidget: Row(
          children: [
            if (numberStr != null)
              Container(
                decoration: BoxDecoration(
                    borderRadius: BorderRadius.circular(30),
                    color: FXColor.main_red),
                alignment: Alignment.center,
                child: Padding(
                  padding: const EdgeInsets.symmetric(horizontal: 4),
                  child: CenterText(
                    numberStr,
                    style: CenterText.content
                        .apply(fontSizeDelta: -4, color: FXColor.main_white),
                  ),
                ),
              ),
            const SizedBox(
              width: 8,
            ),
            ImageUtil.assetImage("images/icon_right_arrow.png",
                width: 16, height: 16),
          ],
        ));
  }

  static Widget _buildShowItem(
      {String? title,
      Function()? onClick,
      Color? textColor,
      bool isRoundRect = false,
      bool isDivide = false,
      bool hasHorizontalPadding = true,
      Widget? rightWidget}) {
    return GestureDetector(
      behavior: HitTestBehavior.translucent,
      onTap: onClick,
      child: Column(
        children: [
          Container(
            color: isRoundRect ? null : FXColor.main_white,
            decoration: isRoundRect
                ? BoxDecoration(
                    borderRadius: BorderRadius.circular(10),
                    color: FXColor.main_white)
                : null,
            padding: hasHorizontalPadding
                ? const EdgeInsets.all(12)
                : const EdgeInsets.symmetric(vertical: 12),
            height: 48,
            child: Row(
              children: [
                FXText(title ?? "",
                    style: FXText.content.apply(color: FXColor.main_black1)),
                const SizedBox(width: 8),
                const Spacer(),
                if (rightWidget != null) rightWidget,
              ],
            ),
          ),
          if (isDivide)
            EditCommonUi.buildDividerItem(
                isMargin: hasHorizontalPadding ? true : false),
        ],
      ),
    );
  }
}
