import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';

class PageLoadingListView extends StatefulWidget {
  final IndexedWidgetBuilder itemBuilder;
  final int itemCount;
  final Function(int pageNum) fetchData;
  final Function() onRefresh;
  final bool Function() hasMoreData;

  const PageLoadingListView(
      {Key? key,
      required this.itemBuilder,
      required this.itemCount,
      required this.fetchData,
      required this.hasMoreData,
      required this.onRefresh})
      : super(key: key);

  @override
  State<StatefulWidget> createState() {
    return PageLoadingListViewState();
  }
}

class PageLoadingListViewState extends State<PageLoadingListView> {
  final ScrollController _scrollController = ScrollController();
  bool isLoading = false;
  int pageNum = 1;
  final GlobalKey<RefreshIndicatorState> _indicatorKey = GlobalKey();

  @override
  void initState() {
    _scrollController.addListener(() {
      if (_scrollController.position.pixels == _scrollController.position.maxScrollExtent) {
        if (isLoading || !widget.hasMoreData()) return;
        fetchData(++pageNum);
      }
    });
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return RefreshIndicator(
      key: _indicatorKey,
      onRefresh: () async {
        if(isLoading) return;
        pageNum = 1;
        refresh();
      },
      child: ListView.builder(
        physics:  const AlwaysScrollableScrollPhysics(),
          controller: _scrollController, itemCount: widget.itemCount, itemBuilder: widget.itemBuilder),
    );
  }

  void fetchData(int pageNum){
    isLoading = true;
    widget.fetchData(pageNum);
    isLoading = false;
  }

  void refresh(){
    isLoading = true;
    widget.onRefresh();
    isLoading = false;
  }

  void forceRefresh() {
    _indicatorKey.currentState?.show();
  }
}
