import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:lib_common_fx/lib_common_fx.dart';
import 'package:lib_common_fx/utils/image_utils.dart';

class CheckBoxDialog extends StatefulWidget {
  final List dataList;
  final String? title;
  int selectId;
  CheckBoxDialog({Key? key, required this.dataList, this.title, required this.selectId}) : super(key: key);

  @override
  CheckBoxDialogState createState() => CheckBoxDialogState();
}

class CheckBoxDialogState extends State<CheckBoxDialog> {

  @override
  Widget build(BuildContext context) {
    return Dialog(
        insetPadding: const EdgeInsets.symmetric(horizontal: 48),
        backgroundColor: FXColor.main_white,
        shape: RoundedRectangleBorder(borderRadius: BorderRadius.circular(10)),
        child: Container(
            constraints: const BoxConstraints(maxHeight: 246, minWidth: 102),
            height: widget.dataList.length * 36 + 58 + 8,
            child: Column(children: [
              if (!FxUtils.isTextEmpty(widget.title))
                Padding(
                  padding: const EdgeInsets.only(left: 24, right: 24, bottom: 6, top: 20),
                  child: FXText(
                    widget.title!,
                    style: FXText.content.apply(fontWeightDelta: 2, color: FXColor.main_black1, fontSizeDelta: 2),
                  ),
                ),
              Expanded(
                  child: ListView.builder(
                      itemCount: widget.dataList.length,
                      itemBuilder: (BuildContext context, int index) {
                        return _checkBoxItem(widget.dataList[index], index);
                      })),
              const SizedBox(height: 8)
            ])));
  }

  Widget _checkBoxItem(BaseEntity item, int index) {
    return GestureDetector(
        onTap: () {
          setState(() {
            widget.selectId = item.getId();
            Navigator.pop(context, item);
          });
        },
        child: Container(
          color: FXColor.main_white,
            padding: const EdgeInsets.only(left: 24, right: 24, bottom: 6,top: 6),
            width: double.infinity,
            child: (Row(children: [
              ImageUtil.assetImage(item.getId() == widget.selectId ? "images/ic_trans_price_change_select.png" : "images/ic_trans_price_change_unselect.png",
                height: 16,
                width: 16,
              ),
              const SizedBox(
                width: 14,
              ),
              FXText(item.getName(), style: FXText.content)
            ]))));
  }
}
