import 'package:flutter/material.dart';
import 'package:lib_common_fx/lib_common_fx.dart';

class ItemDialog extends StatefulWidget {
  final List dataList;
  final String? title;
  int? selectId;
  final ScrollPhysics? physics;
  final Widget? customWidget;
  final int crossAxisCount;
  final double childAspectRatio;
  ItemDialog({Key? key, required this.dataList, this.title,  this.selectId, this.physics, this.customWidget, this.crossAxisCount = 3, this.childAspectRatio = 2.44}) : super(key: key) {

  }

  @override
  ItemDialogState createState() => ItemDialogState();
}

class ItemDialogState extends State<ItemDialog> {
  // static const int crossAxisCount = 3;
  @override
  Widget build(BuildContext context) {
    return Dialog(
        insetPadding: const EdgeInsets.symmetric(horizontal: 20),
        backgroundColor: FXColor.main_white,
        shape: RoundedRectangleBorder(borderRadius: BorderRadius.circular(10)),
        child: Column(mainAxisSize: MainAxisSize.min, children: [
          if (!FxUtils.isTextEmpty(widget.title))
            Padding(
              padding: const EdgeInsets.only(left: 24, right: 24, bottom: 12, top: 20),
              child: CenterText(
                widget.title!,
                style: FXText.content.apply(fontWeightDelta: 2, color: FXColor.main_black1, fontSizeDelta: 2),
              ),
            ),
          GridView.builder(
              physics: widget.physics ?? const NeverScrollableScrollPhysics(),
              padding: const EdgeInsets.symmetric(horizontal: 24),
              gridDelegate: SliverGridDelegateWithFixedCrossAxisCount(
                crossAxisCount: widget.crossAxisCount,
                mainAxisSpacing: 12,
                crossAxisSpacing: 12,
                childAspectRatio: widget.childAspectRatio,
              ),
              itemCount: widget.dataList.length,
              shrinkWrap: true,
              itemBuilder: (BuildContext context, int index) {
                return _itemItem(widget.dataList[index], index);
              }),
          if (widget.customWidget != null) widget.customWidget!,
          const SizedBox(
            height: 24,
          ),
        ]));
  }

  Widget _itemItem(BaseEntity item, int index) {
    return GestureDetector(
        onTap: () {
          setState(() {
            widget.selectId = item.getId();
            Navigator.pop(context,item);
          });
        },
        child: Container(
          margin: const EdgeInsets.only(bottom: 0.5),//解决边框下缘被切边的问题
          alignment: Alignment.center,
            decoration: BoxDecoration(color: item.getId() == widget.selectId ? FXColor.main_blue.withOpacity(0.1) : FXColor.main_white,
                borderRadius: BorderRadius.circular(50),
                border: Border.fromBorderSide(
                    BorderSide(width: 0.5, color: item.getId() == widget.selectId ? FXColor.main_blue : FXColor.main_black5))),
            child: (FXText(item.getName(), style: FXText.content.apply(color: item.getId() == widget.selectId ? FXColor.main_blue:FXColor.main_black1)))));
  }
}
