import 'dart:io';

import 'package:flutter/cupertino.dart';
import 'package:lib_common_fx/utils/fx_colors.dart';


class CenterText extends StatelessWidget {
  final String data;
  final TextStyle style;
  final int? maxLines;
  final double textMaxWidth;
  final TextAlign? textAlign;

  static const content = TextStyle(
      color: FXColor.main_black1,
      fontWeight: FontWeight.normal,
      fontSize: 16);
  static var input_hint = content.copyWith(letterSpacing: 0, color: FXColor.main_black3, height: 1.5);
  static var subtitle = content.apply(fontSizeDelta: -2);
  static var comment = subtitle.apply(color: FXColor.main_black3);
  static var title_black = content.copyWith(fontWeight: FontWeight.w600, fontSize: 18);
  static var title_white = title_black.apply(color: FXColor.main_white);
  static const _fontSize2HeightMap = {
    11: 19,
    12: 20,
    14: 22,
    16: 24,
    18: 26,
    20: 28,
    28: 36,
    48: 56
  };

  static final  _fontSizePaddingMap = {
    11: -1.5,
    12: -1.5,
    14: Platform.isAndroid?-1.0: -2.0,
    16: -1,
    18: -1.3,
    20: -0.5,
    28: -0.3,
    48: 1
  };

  static double? adjustedHeight(double? fontSize) {
    return (_fontSize2HeightMap[fontSize] ?? 24) / (fontSize ?? 16);
  }

  const CenterText(this.data,
      {Key? key, this.style = content, this.maxLines, this.textMaxWidth = double.infinity, this.textAlign})
      : super(key: key);

  @override
  Widget build(BuildContext context) {
    return Container(
      transform: Matrix4.translationValues(0, (_fontSizePaddingMap[style.fontSize]??0).toDouble(), 0),
      child: Text(data,
          overflow: maxLines != null? TextOverflow.ellipsis: null,
          maxLines: maxLines,
          textAlign: textAlign,
          style: style,
          strutStyle: StrutStyle(
            fontSize: style.fontSize,
            fontWeight: style.fontWeight,
            height: adjustedHeight(style.fontSize),
            forceStrutHeight: true, // 关键属性 强制改为文字高度
          )
      ),
    );
  }
}
