import 'package:flutter/material.dart';

import 'bottom_dialog.dart';
import 'fx_colors.dart';
import 'fx_text.dart';

class DatePicker extends StatefulWidget {
  final Null Function(DateTime? result) onConfirm;
  final PickerType type;
  final String title;
  final DateTime? selectDateTime;

  const DatePicker(
      {Key? key,
      required this.onConfirm,
      this.title = "请选择日期",
      this.type = PickerType.yyyyMMdd,
      this.selectDateTime})
      : super(key: key);

  @override
  _DatePickerState createState() => _DatePickerState();
}

class _DatePickerState extends State<DatePicker> {
  DateTime resultDate = DateTime.now();
  final GlobalKey<_DateSelectorState> _dateSelectorKey = GlobalKey();

  bool changeStart = true;

  @override
  Widget build(BuildContext context) {
    var dateSelector = DateSelector(
        key: _dateSelectorKey,
        type: widget.type,
        selectDateTime: widget.selectDateTime ?? DateTime.now(),
        onSelect: (dateTime) {
          setState(() {
            resultDate = dateTime;
          });
        });

    return BottomDialog<DateTime>(
        getSelectedItem: () {
          return resultDate;
        },
        onConfirm: widget.onConfirm,
        canConfirm: () {
          return true;
        },
        title: widget.title,
        body: Column(
          children: [dateSelector],
        ));
  }
}

class DateSelector extends StatefulWidget {
  final Function(DateTime dateTime)? onSelect;
  final PickerType type;
  final DateTime selectDateTime;

  const DateSelector(
      {Key? key,
      this.onSelect,
      this.type = PickerType.yyyyMMdd,
      required this.selectDateTime})
      : super(key: key);

  @override
  _DateSelectorState createState() => _DateSelectorState();

  static _DateSelectorState? of(BuildContext context) {
    return context.findAncestorStateOfType<_DateSelectorState>();
  }
}

class _DateSelectorState extends State<DateSelector> {
  DateTime get date => DateTime(year, month, day, hour, minute, second);

  var year;
  var month;
  var day;
  var hour;
  var minute;
  var second;

  late final _yearController;
  late final _monthController;
  late final _dayController;
  late final _hourController;
  late final _minuteController;
  late final _secondController;

  @override
  void initState() {
    year = hasYear(widget.type) ? widget.selectDateTime.year : 2020;
    month = hasMonth(widget.type) ? widget.selectDateTime.month : 1;
    day = hasDay(widget.type) ? widget.selectDateTime.day : 1;
    hour = hasHour(widget.type) ? widget.selectDateTime.hour : 0;
    minute = hasMinute(widget.type) ? widget.selectDateTime.minute : 0;
    second = hasSecond(widget.type) ? widget.selectDateTime.second : 0;

    _yearController = PageController(
        viewportFraction: 0.2, initialPage: widget.selectDateTime.year - 2020);
    _monthController = PageController(
        viewportFraction: 0.2, initialPage: widget.selectDateTime.month - 1);
    _dayController = PageController(
        viewportFraction: 0.2, initialPage: widget.selectDateTime.day - 1);
    _hourController = PageController(
        viewportFraction: 0.2, initialPage: widget.selectDateTime.hour);
    _minuteController = PageController(
        viewportFraction: 0.2, initialPage: widget.selectDateTime.minute);
    _secondController = PageController(
        viewportFraction: 0.2, initialPage: widget.selectDateTime.second);
    super.initState();
  }

  @override
  void dispose() {
    _yearController.dispose();
    _monthController.dispose();
    _dayController.dispose();
    _hourController.dispose();
    _minuteController.dispose();
    _secondController.dispose();

    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return Expanded(
      child: Stack(
        children: [
          Align(
              alignment: Alignment.center,
              child: Container(
                height: 48,
                decoration: BoxDecoration(
                    color: FXColor.main_blue.withOpacity(0.08),
                    border: const Border(
                        top: BorderSide(color: FXColor.main_blue),
                        bottom: BorderSide(color: FXColor.main_blue))),
              )),
          Padding(
            padding: const EdgeInsets.symmetric(horizontal: 24),
            child: Row(
              mainAxisAlignment: MainAxisAlignment.end,
              children: [
                if (hasYear(widget.type))
                  Expanded(
                    child: PageView.builder(
                        itemCount: 20,
                        scrollDirection: Axis.vertical,
                        physics: const BouncingScrollPhysics(),
                        controller: _yearController,
                        onPageChanged: (index) {
                          setState(() {
                            year = index + 2020;
                            widget.onSelect?.call(date);
                          });
                        },
                        itemBuilder: (context, index) {
                          return Align(
                              alignment: Alignment.centerLeft,
                              child: Text('${index + 2020}年',
                                  style: year - 2020 == index
                                      ? FXText.content
                                      : FXText.content
                                          .apply(color: FXColor.black)));
                        }),
                  ),
                if (hasMonth(widget.type))
                  Expanded(
                    child: PageView.builder(
                        itemCount: 12,
                        scrollDirection: Axis.vertical,
                        physics: const BouncingScrollPhysics(),
                        controller: _monthController,
                        onPageChanged: (index) {
                          setState(() {
                            month = index + 1;
                            widget.onSelect?.call(date);
                          });
                        },
                        itemBuilder: (context, index) {
                          return Align(
                              child: Text('${index + 1}月',
                                  style: month - 1 == index
                                      ? FXText.content
                                      : FXText.content
                                          .apply(color: FXColor.black)));
                        }),
                  ),
                if (hasDay(widget.type))
                  Expanded(
                    child: PageView.builder(
                        itemCount: DateUtils.getDaysInMonth(year, month),
                        scrollDirection: Axis.vertical,
                        physics: const BouncingScrollPhysics(),
                        controller: _dayController,
                        onPageChanged: (index) {
                          setState(() {
                            day = index + 1;
                            widget.onSelect?.call(date);
                          });
                        },
                        itemBuilder: (context, index) {
                          return Align(
                              alignment: Alignment.centerRight,
                              child: Text('${index + 1}日',
                                  style: day - 1 == index
                                      ? FXText.content
                                      : FXText.content
                                          .apply(color: FXColor.black)));
                        }),
                  ),
                if (hasHour(widget.type))
                  Expanded(
                    child: PageView.builder(
                        itemCount: 24,
                        scrollDirection: Axis.vertical,
                        physics: const BouncingScrollPhysics(),
                        controller: _hourController,
                        onPageChanged: (index) {
                          setState(() {
                            hour = index;
                            widget.onSelect?.call(date);
                          });
                        },
                        itemBuilder: (context, index) {
                          return Align(
                              alignment: Alignment.centerRight,
                              child: Text('$index时',
                                  style: hour == index
                                      ? FXText.content
                                      : FXText.content
                                          .apply(color: FXColor.black)));
                        }),
                  ),
                if (hasMinute(widget.type))
                  Expanded(
                    child: PageView.builder(
                        itemCount: 60,
                        scrollDirection: Axis.vertical,
                        physics: const BouncingScrollPhysics(),
                        controller: _minuteController,
                        onPageChanged: (index) {
                          setState(() {
                            minute = index;
                            widget.onSelect?.call(date);
                          });
                        },
                        itemBuilder: (context, index) {
                          return Align(
                              alignment: Alignment.centerRight,
                              child: Text('$index分',
                                  style: minute == index
                                      ? FXText.content
                                      : FXText.content
                                          .apply(color: FXColor.black)));
                        }),
                  ),
                if (hasSecond(widget.type))
                  Expanded(
                    child: PageView.builder(
                        itemCount: 60,
                        scrollDirection: Axis.vertical,
                        physics: const BouncingScrollPhysics(),
                        controller: _secondController,
                        onPageChanged: (index) {
                          setState(() {
                            second = index;
                            widget.onSelect?.call(date);
                          });
                        },
                        itemBuilder: (context, index) {
                          return Align(
                              alignment: Alignment.centerRight,
                              child: Text('$index秒',
                                  style: second == index
                                      ? FXText.content
                                      : FXText.content
                                          .apply(color: FXColor.black)));
                        }),
                  ),
              ],
            ),
          )
        ],
      ),
    );
  }

  hasYear(PickerType type) {
    return type == PickerType.yyyy ||
        type == PickerType.yyyyMM ||
        type == PickerType.yyyyMMdd ||
        type == PickerType.yyyyMMddHH ||
        type == PickerType.yyyyMMddHHmm ||
        type == PickerType.yyyyMMddHHmmss;
  }

  hasMonth(PickerType type) {
    return type == PickerType.yyyyMM ||
        type == PickerType.yyyyMMdd ||
        type == PickerType.yyyyMMddHH ||
        type == PickerType.yyyyMMddHHmm ||
        type == PickerType.yyyyMMddHHmmss;
  }

  hasDay(PickerType type) {
    return type == PickerType.yyyyMMdd ||
        type == PickerType.yyyyMMddHH ||
        type == PickerType.yyyyMMddHHmm ||
        type == PickerType.yyyyMMddHHmmss;
  }

  hasHour(PickerType type) {
    return type == PickerType.yyyyMMddHH ||
        type == PickerType.yyyyMMddHHmm ||
        type == PickerType.yyyyMMddHHmmss;
  }

  hasMinute(PickerType type) {
    return type == PickerType.yyyyMMddHHmm || type == PickerType.yyyyMMddHHmmss;
  }

  hasSecond(PickerType type) {
    return type == PickerType.yyyyMMddHHmmss;
  }
}

enum PickerType {
  yyyy,
  yyyyMM,
  yyyyMMdd,
  yyyyMMddHH,
  yyyyMMddHHmm,
  yyyyMMddHHmmss,
}
