import 'dart:ffi';

import 'package:flutter/material.dart';
import 'package:lib_common_fx/utils/image_utils.dart';

import 'center_text.dart';
import 'fx_colors.dart';
import 'fx_text.dart';
import 'fx_utils.dart';

class FxDialog extends StatefulWidget {
  final String? imageName;
  final String? title;
  final String? message;
  final String? confirmStr;
  final String? cancelStr;
  final Function()? onConfirm;
  final Function()? onCancel;
  final Widget? customContent;
  final bool Function()? dismissDialogWhenConfirmClick;
  final TextAlign? messageAlign; //message对齐方式，默认居中，
  final double? messageHeightDelta; //message额外的行间距，字体为14默认为0.57,字体为16默认为0.5

  const FxDialog(
      {Key? key,
      this.title,
      this.onConfirm,
      this.onCancel,
      required this.message,
      this.cancelStr,
      this.confirmStr,
      this.customContent,
      this.imageName,
      this.messageAlign,
      this.dismissDialogWhenConfirmClick,
      this.messageHeightDelta})
      : super(key: key);

  @override
  _FxDialogState createState() => _FxDialogState();

  static Future<T?> show<T>(BuildContext context,
      {String? title,
      Function()? onConfirm,
      Function()? onCancel,
      String? message,
      String? cancelStr,
      String? confirmStr,
      Widget? customContent,
      String? imageName,
      TextAlign? messageAlign,
      double? messageHeightDelta,
      bool Function()? dismissDialogWhenConfirmClick,
      bool barrierDismissible = true //点击空白能否消失，false不消失
      }) {
    return showDialog<T>(
        context: context,
        barrierDismissible: barrierDismissible,
        barrierColor: FXColor.black.withOpacity(0.4),
        builder: (BuildContext context) {
          return FxDialog(
            imageName: imageName,
            title: title,
            message: message,
            confirmStr: confirmStr,
            cancelStr: cancelStr,
            onConfirm: onConfirm,
            onCancel: onCancel,
            customContent: customContent,
            messageAlign: messageAlign,
            messageHeightDelta: messageHeightDelta,
            dismissDialogWhenConfirmClick: dismissDialogWhenConfirmClick,
          );
        });
  }
}

class _FxDialogState extends State<FxDialog> {
  @override
  Widget build(BuildContext context) {
    return Dialog(
      insetPadding: const EdgeInsets.symmetric(horizontal: 48),
      backgroundColor: FXColor.main_white,
      elevation: 0,
      shape: RoundedRectangleBorder(borderRadius: BorderRadius.circular(10)),
      child: Column(
        mainAxisSize: MainAxisSize.min,
        children: [
          const SizedBox(height: 20),
          if (!FxUtils.isTextEmpty(widget.imageName))
            Padding(
              padding: const EdgeInsets.only(left: 24, top: 0, right: 24, bottom: 12),
              child: ImageUtil.assetImage(
                widget.imageName!,
                width: 40,
                height: 40,
              ),
            ),
          if (!FxUtils.isTextEmpty(widget.title))
            Padding(
              padding: const EdgeInsets.only(left: 24, top: 0, right: 24, bottom: 12),
              child: CenterText(
                widget.title!,
                style: CenterText.content.apply(fontWeightDelta: 2, color: FXColor.main_black1, fontSizeDelta: 2),
              ),
            ),
          if (!FxUtils.isTextEmpty(widget.message))
            Padding(
              padding: const EdgeInsets.only(left: 24, top: 0, right: 24, bottom: 20),
              child: SizedBox(
                width: double.infinity,
                child: Column(
                  crossAxisAlignment: CrossAxisAlignment.start,
                  children: [
                    Container(
                      constraints: const BoxConstraints(minWidth: double.infinity),
                      child: CenterText(widget.message!,
                          textAlign: widget.messageAlign ?? TextAlign.center,
                          style: widget.imageName != null
                              ? CenterText.content.apply(
                                  color: FXColor.main_black2,
                                  heightDelta: widget.messageHeightDelta ?? 0.57,
                                  fontSizeDelta: -2)
                              : CenterText.content.apply(heightDelta: widget.messageHeightDelta ?? 0.5)),
                    ),
                  ],
                ),
              ),
            ),
          if (widget.customContent != null) widget.customContent!,
          Container(height: 0.5, color: FXColor.main_black5),
          IntrinsicHeight(
            child: Row(
              children: [
                if (!FxUtils.isTextEmpty(widget.cancelStr))
                  Expanded(
                    child: GestureDetector(
                      onTap: () {
                        Navigator.pop(context);
                        widget.onCancel?.call();
                      },
                      behavior: HitTestBehavior.opaque,
                      child: Container(
                        alignment: Alignment.center,
                        padding: const EdgeInsets.symmetric(vertical: 16.0),
                        child:
                            CenterText(widget.cancelStr!, style: CenterText.content.apply(color: FXColor.main_black2)),
                      ),
                    ),
                  ),
                if (!FxUtils.isTextEmpty(widget.cancelStr)) Container(width: 0.5, color: FXColor.main_black5),
                if (!FxUtils.isTextEmpty(widget.confirmStr))
                  Expanded(
                    child: GestureDetector(
                      onTap: () {
                        if (widget.dismissDialogWhenConfirmClick == null || widget.dismissDialogWhenConfirmClick!()) {
                          Navigator.pop(context);
                        }
                        widget.onConfirm?.call();
                      },
                      behavior: HitTestBehavior.opaque,
                      child: Container(
                          alignment: Alignment.center,
                          padding: const EdgeInsets.symmetric(vertical: 16.0),
                          child: CenterText(widget.confirmStr!,
                              style: CenterText.content.apply(color: FXColor.main_blue))),
                    ),
                  ),
              ],
            ),
          )
        ],
      ),
    );
  }
}
