import 'package:decimal/decimal.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';

///当输入数字时，用于限制输入的位数；
/// TextField(
///   inputFormatters: [
///     //限制整数长度100，小数长度2
///     FXNumberInputFormatter(integerLength: 100, decimalLength: 2)
///   ],
/// );
///
class FXNumberInputFormatter extends TextInputFormatter {
  int decimalLength;
  int integerLength;
  bool allowInputDecimal = true;

  FXNumberInputFormatter({
    this.integerLength = 100,
    this.decimalLength = 2,
  }) : super();

  @override
  TextEditingValue formatEditUpdate(
      TextEditingValue oldValue, TextEditingValue newValue) {
    String value = newValue.text;
    int selectionIndex = newValue.selection.end;
    if (newValue.text.contains('.')) {
      int pointIndex = newValue.text.indexOf('.');
      String beforePoint = newValue.text.substring(0, pointIndex);
      //小数点前内容大于integerLength
      if (beforePoint.length > integerLength) {
        value = oldValue.text;
        selectionIndex = oldValue.selection.end;
      } else {
        //小数点前内容小于等于integerLength
        String afterPoint =
            newValue.text.substring(pointIndex + 1, newValue.text.length);
        if (afterPoint.length > decimalLength) {
          value = oldValue.text;
          selectionIndex = oldValue.selection.end;
        }
      }
    } else {
      if (newValue.text.length > integerLength) {
        value = oldValue.text;
        selectionIndex = oldValue.selection.end;
      }
    }
    return TextEditingValue(
      text: value,
      selection: TextSelection.collapsed(offset: selectionIndex),
    );
  }
}
