// ignore_for_file: prefer_const_constructors_in_immutables, must_be_immutable

import 'package:flutter/material.dart';
import 'package:lib_common_fx/lib_common_fx.dart';

///Author: wangys
///Date: 2022-10-17 09:30:41
///Description: 可点击文字

// void test() {
//   //设置黑色可点击的字体,内部边距
//   FxTextButton(
//     text: 'Button',
//     textStyle: const TextStyle(fontSize: 16, color: Colors.black),
//     padding: const MaterialStatePropertyAll(EdgeInsets.fromLTRB(12, 8, 12, 8)),
//     onPressed: () {},
//   );
//   //设置文字点击变色，不要设置textStyle中的颜色
//   FxTextButton(
//     text: 'Button',
//     textStyle: TextStyle(fontSize: 16),
//     foregroundColor: MaterialStateProperty.resolveWith((states) {
//       if (states.contains(MaterialState.disabled)) {
//         return FXColor.main_black3;
//       } else {
//         return FXColor.main_black1;
//       }
//     }),
//     onPressed: () {},
//   );
// }

class FxTextButton extends StatelessWidget {
  FxTextButton({
    Key? key,
    required this.text,
    required this.onPressed,
    this.textStyle,
    this.background,
    this.foreground,
    this.padding,
    this.overlay,
    this.shape,
    this.textStyleProperty,
    this.backgroundProperty,
    this.foregroundProperty,
    this.paddingProperty,
    this.overlayProperty,
    this.shapeProperty,
  }) : super(key: key);

  final void Function()? onPressed;
  final String text;

  TextStyle? textStyle;
  Color? background;
  Color? foreground;
  EdgeInsets? padding;
  Color? overlay;
  OutlinedBorder? shape;

  MaterialStateProperty<TextStyle?>? textStyleProperty;
  MaterialStateProperty<Color?>? backgroundProperty;
  MaterialStateProperty<Color?>? foregroundProperty;
  MaterialStateProperty<EdgeInsets?>? paddingProperty;
  MaterialStateProperty<Color?>? overlayProperty;
  MaterialStateProperty<OutlinedBorder?>? shapeProperty;

  ///扩大点击区域请设置padding
  ///默认FXText.content样式
  FxTextButton.text({
    super.key,
    this.onPressed,
    required this.text,
    this.textStyle = FXText.content,
    this.background = Colors.transparent,
    this.foreground,
    this.padding = const EdgeInsets.fromLTRB(0, 8, 0, 8),
    this.overlay = Colors.transparent,
    this.shape,
  });

  @override
  Widget build(BuildContext context) {
    //字体样式
    var resolvedTextStyle = textStyle != null
        ? MaterialStatePropertyAll(textStyle)
        : textStyleProperty;

    //前景色，字体颜色
    MaterialStateProperty<Color?>? resolvedForegroundColor;
    //有限使用foreground、其次foregroundProperty、其次textStyle!.color
    if (foreground != null) {
      resolvedForegroundColor = MaterialStatePropertyAll(foreground);
    } else if (foregroundProperty != null) {
      resolvedForegroundColor = foregroundProperty;
    } else if (textStyle != null && textStyle!.color != null) {
      resolvedForegroundColor = MaterialStatePropertyAll(textStyle!.color);
    }

    //背景色
    var resolvedBackgroundColor = background != null
        ? MaterialStatePropertyAll(background)
        : backgroundProperty;

    //内边距
    var resolvedPadding =
        padding != null ? MaterialStatePropertyAll(padding) : paddingProperty;

    //水波纹
    var resolvedOverlayColor =
        overlay != null ? MaterialStatePropertyAll(overlay) : overlayProperty;

    //外形
    var resolvedShape =
        shape != null ? MaterialStatePropertyAll(shape) : shapeProperty;

    var style = ButtonStyle(
      backgroundColor: resolvedBackgroundColor,
      foregroundColor: resolvedForegroundColor,
      textStyle: resolvedTextStyle,
      padding: resolvedPadding,
      minimumSize: MaterialStateProperty.all(const Size(0, 0)),
      maximumSize: MaterialStateProperty.all(
          const Size(double.infinity, double.infinity)),
      overlayColor: resolvedOverlayColor,
      animationDuration: const Duration(milliseconds: 50),
      shape: resolvedShape,
      tapTargetSize: MaterialTapTargetSize.shrinkWrap,
    );

    return TextButton(
      onPressed: onPressed,
      style: style,
      child: Text(text),
    );
  }
}
